package envoy.util;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;

/**
 * Project: <strong>envoy-client</strong><br>
 * File: <strong>SerializationUtils.java</strong><br>
 * Created: <strong>23.12.2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @since Envoy Common v0.2-alpha
 */
public class SerializationUtils {

	private static byte[] intToBytes(int n) { return new byte[] { (byte) (n >>> 24), (byte) (n >>> 16), (byte) (n >>> 8), (byte) n }; }

	/**
	 * Deserializes an arbitrary {@link Serializable} object from a file.
	 *
	 * @param <T>             the type of the object to deserialize
	 * @param file            the file to deserialize from
	 * @param serializedClass the class of the object to deserialize
	 * @return the deserialized object
	 * @throws IOException            if something failed while deserializing the
	 *                                object
	 * @throws ClassNotFoundException if the deserialized object can not be linked
	 *                                to a class
	 * @since Envoy Common v0.2-alpha
	 */
	public static <T extends Serializable> T read(File file, Class<T> serializedClass) throws IOException, ClassNotFoundException {
		if (file == null) throw new NullPointerException("File is null");
		return read(new FileInputStream(file), serializedClass);
	}

	/**
	 * @param <T>             the deserialized object
	 * @param in              the {@link InputStream} of a serialized Object
	 * @param serializedClass the object type to convert the deserialized object
	 *                        into
	 * @return the deserialized object
	 * @throws IOException            if something failed while deserializing the
	 *                                object
	 * @throws ClassNotFoundException if the deserialized object can not be linked
	 *                                to a class
	 * @since Envoy Common v0.2-alpha
	 */
	public static <T extends Serializable> T read(InputStream in, Class<T> serializedClass) throws IOException, ClassNotFoundException {
		try (ObjectInputStream oin = new ObjectInputStream(in)) {
			return serializedClass.cast(oin.readObject());
		}
	}

	/**
	 * Serializes an arbitrary object to a file.
	 *
	 * @param file the file to serialize to
	 * @param obj  the object to serialize
	 * @throws IOException if an error occurred during serialization
	 * @since Envoy Common v0.2-alpha
	 */
	public static void write(File file, Object obj) throws IOException {
		if (file == null) throw new NullPointerException("File is null");
		if (obj == null) throw new NullPointerException("Object to serialize is null");
		if (!file.exists()) {
			file.getParentFile().mkdirs();
			file.createNewFile();
		}
		try (ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(file))) {
			out.writeObject(obj);
		}
	}

	/**
	 * Serializes an object and writes it into an output stream preceded by 4 bytes
	 * containing the number of serialized bytes.
	 *
	 * @param obj the object to serialize
	 * @param out the output stream to serialize to
	 * @throws IOException if an error occurred during serialization
	 */
	public static void writeBytesWithLength(Object obj, OutputStream out) throws IOException {
		// Serialize object to byte array
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try (ObjectOutputStream oout = new ObjectOutputStream(baos)) {
			oout.writeObject(obj);
		}
		byte[] objBytes = baos.toByteArray();

		// Get length of byte array in bytes
		byte[] objLen = intToBytes(objBytes.length);

		// Write length and byte array
		out.write(objLen);
		out.write(objBytes);
	}

	private SerializationUtils() {}
}